@@ -83,8 +83,8 @@ static int mpl2_read_header(AVFormatContext *s)
     if (!st)
         return AVERROR(ENOMEM);
     avpriv_set_pts_info(st, 64, 1, 10);
-    st->codec->codec_type = AVMEDIA_TYPE_SUBTITLE;
-    st->codec->codec_id   = AV_CODEC_ID_MPL2;
+    st->codecpar->codec_type = AVMEDIA_TYPE_SUBTITLE;
+    st->codecpar->codec_id   = AV_CODEC_ID_MPL2;
 
     while (!avio_feof(s->pb)) {
         char line[4096];
@@ -111,7 +111,7 @@ static int mpl2_read_header(AVFormatContext *s)
         }
     }
 
-    ff_subtitles_queue_finalize(&mpl2->q);
+    ff_subtitles_queue_finalize(s, &mpl2->q);
     return res;
 }
 
