@@ -107,8 +107,11 @@ static int vqf_read_header(AVFormatContext *s)
 
     header_size = avio_rb32(s->pb);
 
-    st->codec->codec_type = AVMEDIA_TYPE_AUDIO;
-    st->codec->codec_id   = AV_CODEC_ID_TWINVQ;
+    if (header_size < 0)
+        return AVERROR_INVALIDDATA;
+
+    st->codecpar->codec_type = AVMEDIA_TYPE_AUDIO;
+    st->codecpar->codec_id   = AV_CODEC_ID_TWINVQ;
     st->start_time = 0;
 
     do {
@@ -120,7 +123,7 @@ static int vqf_read_header(AVFormatContext *s)
 
         len = avio_rb32(s->pb);
 
-        if ((unsigned) len > INT_MAX/2) {
+        if ((unsigned) len > INT_MAX/2 || header_size < 8) {
             av_log(s, AV_LOG_ERROR, "Malformed header\n");
             return -1;
         }
@@ -129,21 +132,18 @@ static int vqf_read_header(AVFormatContext *s)
 
         switch(chunk_tag){
         case MKTAG('C','O','M','M'):
-            if (len < 12)
-                return AVERROR_INVALIDDATA;
-
             avio_read(s->pb, comm_chunk, 12);
-            st->codec->channels = AV_RB32(comm_chunk    ) + 1;
+            st->codecpar->channels = AV_RB32(comm_chunk    ) + 1;
             read_bitrate        = AV_RB32(comm_chunk + 4);
             rate_flag           = AV_RB32(comm_chunk + 8);
             avio_skip(s->pb, len-12);
 
-            if (st->codec->channels <= 0) {
+            if (st->codecpar->channels <= 0) {
                 av_log(s, AV_LOG_ERROR, "Invalid number of channels\n");
                 return AVERROR_INVALIDDATA;
             }
 
-            st->codec->bit_rate              = read_bitrate*1000;
+            st->codecpar->bit_rate = read_bitrate * 1000;
             break;
         case MKTAG('D','S','I','Z'): // size of compressed data
         {
@@ -172,32 +172,32 @@ static int vqf_read_header(AVFormatContext *s)
         av_log(s, AV_LOG_ERROR, "COMM tag not found!\n");
         return -1;
     case 44:
-        st->codec->sample_rate = 44100;
+        st->codecpar->sample_rate = 44100;
         break;
     case 22:
-        st->codec->sample_rate = 22050;
+        st->codecpar->sample_rate = 22050;
         break;
     case 11:
-        st->codec->sample_rate = 11025;
+        st->codecpar->sample_rate = 11025;
         break;
     default:
         if (rate_flag < 8 || rate_flag > 44) {
             av_log(s, AV_LOG_ERROR, "Invalid rate flag %d\n", rate_flag);
             return AVERROR_INVALIDDATA;
         }
-        st->codec->sample_rate = rate_flag*1000;
+        st->codecpar->sample_rate = rate_flag*1000;
         break;
     }
 
-    if (read_bitrate / st->codec->channels <  8 ||
-        read_bitrate / st->codec->channels > 48) {
+    if (read_bitrate / st->codecpar->channels <  8 ||
+        read_bitrate / st->codecpar->channels > 48) {
         av_log(s, AV_LOG_ERROR, "Invalid bitrate per channel %d\n",
-               read_bitrate / st->codec->channels);
+               read_bitrate / st->codecpar->channels);
         return AVERROR_INVALIDDATA;
     }
 
-    switch (((st->codec->sample_rate/1000) << 8) +
-            read_bitrate/st->codec->channels) {
+    switch (((st->codecpar->sample_rate/1000) << 8) +
+            read_bitrate/st->codecpar->channels) {
     case (11<<8) + 8 :
     case (8 <<8) + 8 :
     case (11<<8) + 10:
@@ -214,17 +214,17 @@ static int vqf_read_header(AVFormatContext *s)
         size = 2048;
         break;
     default:
-        av_log(s, AV_LOG_ERROR, "Mode not suported: %d Hz, %d kb/s.\n",
-               st->codec->sample_rate, st->codec->bit_rate);
+        av_log(s, AV_LOG_ERROR, "Mode not supported: %d Hz, %"PRId64" kb/s.\n",
+               st->codecpar->sample_rate, (int64_t)st->codecpar->bit_rate);
         return -1;
     }
-    c->frame_bit_len = st->codec->bit_rate*size/st->codec->sample_rate;
-    avpriv_set_pts_info(st, 64, size, st->codec->sample_rate);
+    c->frame_bit_len = st->codecpar->bit_rate*size/st->codecpar->sample_rate;
+    avpriv_set_pts_info(st, 64, size, st->codecpar->sample_rate);
 
     /* put first 12 bytes of COMM chunk in extradata */
-    if (ff_alloc_extradata(st->codec, 12))
+    if (ff_alloc_extradata(st->codecpar, 12))
         return AVERROR(ENOMEM);
-    memcpy(st->codec->extradata, comm_chunk, 12);
+    memcpy(st->codecpar->extradata, comm_chunk, 12);
 
     ff_metadata_conv_ctx(s, NULL, vqf_metadata_conv);
 
@@ -249,7 +249,7 @@ static int vqf_read_packet(AVFormatContext *s, AVPacket *pkt)
     ret = avio_read(s->pb, pkt->data+2, size);
 
     if (ret != size) {
-        av_free_packet(pkt);
+        av_packet_unref(pkt);
         return AVERROR(EIO);
     }
 
@@ -268,7 +268,7 @@ static int vqf_read_seek(AVFormatContext *s,
     int64_t pos;
 
     st = s->streams[stream_index];
-    pos = av_rescale_rnd(timestamp * st->codec->bit_rate,
+    pos = av_rescale_rnd(timestamp * st->codecpar->bit_rate,
                          st->time_base.num,
                          st->time_base.den * (int64_t)c->frame_bit_len,
                          (flags & AVSEEK_FLAG_BACKWARD) ?
@@ -276,7 +276,7 @@ static int vqf_read_seek(AVFormatContext *s,
     pos *= c->frame_bit_len;
 
     st->cur_dts = av_rescale(pos, st->time_base.den,
-                             st->codec->bit_rate * (int64_t)st->time_base.num);
+                             st->codecpar->bit_rate * (int64_t)st->time_base.num);
 
     if ((ret = avio_seek(s->pb, ((pos-7) >> 3) + s->internal->data_offset, SEEK_SET)) < 0)
         return ret;
